import React, { Component, MouseEvent } from 'react';
export declare type SelectionMarker = {
    selectionStart: number;
    selectionLength: number;
};
interface InteractiveHighlighterProps {
    text: string;
    customClass?: string;
    customClassFn?(highlightIndexes: number[]): string;
    highlights: {
        startIndex: number;
        numChars: number;
    }[];
    selectionHandler?(selection: string, selectionStart: number, selectionLen: number): void;
    getSelectionFn?(): SelectionMarker;
}
/**
 * InteractiveHighlighter component.
 *
 * Allows highlighting of the text selected by user with given custom class (or default)
 * and calls optional callback function with the following selection details:
 * - selected text
 * - selection start index
 * - selection end index
 *
 * Supports pre-existing highlights passed through the "highlights" property.
 *
 * It is difficult to unit test the DOM-based selection API, so this class supports a property
 * providing a mock that can be used to return the start & length of a selection. This is
 * intended only for use in tests.
 */
export declare class InteractiveHighlighter extends Component<InteractiveHighlighterProps, {}> {
    private _getSelectionMarker;
    private _customClassFn;
    private _segments;
    constructor(props: InteractiveHighlighterProps);
    private _getSelectionFromDOM;
    private _computeSegments;
    onMouseUpHandler(e: MouseEvent): void;
    render(): React.ReactNode;
}
export {};
