"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
// true if arrays consist of the same primitives (in the same order)
function arraysEqual(a, b) {
    if (a === b)
        return true;
    if (a == null || b == null)
        return false;
    if (a.length !== b.length)
        return false;
    for (var i = 0; i < a.length; ++i) {
        if (a[i] !== b[i])
            return false;
    }
    return true;
}
/**
 * InteractiveHighlighter component.
 *
 * Allows highlighting of the text selected by user with given custom class (or default)
 * and calls optional callback function with the following selection details:
 * - selected text
 * - selection start index
 * - selection end index
 *
 * Supports pre-existing highlights passed through the "highlights" property.
 *
 * It is difficult to unit test the DOM-based selection API, so this class supports a property
 * providing a mock that can be used to return the start & length of a selection. This is
 * intended only for use in tests.
 */
var InteractiveHighlighter = /** @class */ (function (_super) {
    __extends(InteractiveHighlighter, _super);
    function InteractiveHighlighter(props) {
        var _this = _super.call(this, props) || this;
        if (_this.props.getSelectionFn) {
            _this._getSelectionMarker = _this.props.getSelectionFn;
        }
        else {
            _this._getSelectionMarker = _this._getSelectionFromDOM;
        }
        _this.onMouseUpHandler = _this.onMouseUpHandler.bind(_this);
        if (_this.props.customClassFn) {
            _this._customClassFn = _this.props.customClassFn;
        }
        else {
            _this._customClassFn = function (highlightIndexes) {
                if (highlightIndexes.length > 0) {
                    return _this.props.customClass || "default";
                }
                return "";
            };
        }
        _this._segments = [];
        return _this;
    }
    InteractiveHighlighter.prototype._getSelectionFromDOM = function () {
        var selectionObj = window.getSelection();
        if (!selectionObj) {
            console.error("getSelection() returned null");
            return null;
        }
        // anchorNode is the node in which the selection begins; focusNode that
        //  in which it ends. (See https://developer.mozilla.org/en-US/docs/Web/API/Selection.)
        var selection = selectionObj.toString();
        var anchorNode = selectionObj.anchorNode;
        var focusNode = selectionObj.focusNode;
        var anchorOffset = selectionObj.anchorOffset;
        var focusOffset = selectionObj.focusOffset;
        // did we select "forward" or backward?
        var relativePosition;
        var isForwardSelection = false;
        if (anchorNode && focusNode) { // wrapped to stop compiler warnings about possible null-ness
            relativePosition = anchorNode.compareDocumentPosition(focusNode);
        }
        else {
            console.warn("Returning from onMouseUpHandler because anchorNode or focusNode are null");
            return null;
        }
        /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
        if (relativePosition === anchorNode.DOCUMENT_POSITION_FOLLOWING) {
            isForwardSelection = true;
        }
        else if (relativePosition === 0) {
            isForwardSelection = (focusOffset - anchorOffset) > 0;
        }
        var selectionStart = isForwardSelection ? anchorOffset : focusOffset;
        // figure out the actual absolute index of the selection (in the original string) - to do
        //  this we need to account for the spans that might have been rendered in order to handle
        //  highlights that already exist.
        var parent;
        if (isForwardSelection) {
            if (anchorNode.parentNode) {
                parent = anchorNode.parentNode;
            }
            else {
                console.error("Ended up with null anchorNode.parentNode - should be impossible!");
                return null;
            }
        }
        else {
            if (focusNode.parentNode) {
                parent = focusNode.parentNode;
            }
            else {
                console.error("Ended up with null focusNode.parentNode - should be impossible!");
                return null;
            }
        }
        // adjust selectionStart to reflect the index into the whole text (not just the current segment)
        if (parent.getAttribute('data-segment')) {
            var segmentNum = Number(parent.getAttribute('data-segment'));
            selectionStart += this._segments[segmentNum].start;
        }
        // clear selection
        selectionObj.removeAllRanges();
        return { selectionStart: selectionStart, selectionLength: selection.length };
    };
    // Note: a "segment" is a string in which each char is highlighted the same way. When there are no
    //  overlapping highlights, segments are simply highlights and the non-highlighted text
    //  between them. When there are overlapping highlights, segments distinguish text
    //  in highlight A from text in highlights A and B from text in highlight C (and so forth).
    InteractiveHighlighter.prototype._computeSegments = function () {
        var highlights = this.props.highlights;
        var segments = [];
        var segmentStart = 0;
        var currentHighlights = [];
        var _loop_1 = function (idx) {
            // build an array of booleans corresponding to each highlight - true means
            //  the current character is in the given highlight
            var highlightsIncludingThisChar = highlights.map(function (h) { return (idx >= Number(h.startIndex)) && (idx < Number(h.startIndex) + Number(h.numChars)) ? true : false; });
            if (!arraysEqual(currentHighlights, highlightsIncludingThisChar)) {
                // the current character is the start of the next segment
                if (idx > 0) {
                    segments.push({
                        start: segmentStart,
                        end: idx - 1,
                        // the highlights array has a number for each highlight this segment is part of
                        // (this uses -1 to satisfy the compiler - null might make more sense)
                        highlights: currentHighlights.map(function (e, i) { return e ? i : -1; }).filter(function (e) { return e !== -1; })
                    });
                    segmentStart = idx;
                }
                currentHighlights = highlightsIncludingThisChar.slice();
            }
        };
        for (var idx = 0; idx < this.props.text.length; idx++) {
            _loop_1(idx);
        }
        if (segments.length === 0) {
            // special case - no highlights
            segments.push({ start: 0, end: this.props.text.length, highlights: [] });
        }
        else {
            // record the final segment
            segments.push({
                start: segmentStart,
                end: this.props.text.length,
                highlights: currentHighlights.map(function (e, i) { return e ? i : -1; }).filter(function (e) { return e !== -1; })
            });
        }
        return (segments);
    };
    InteractiveHighlighter.prototype.onMouseUpHandler = function (e) {
        e.preventDefault();
        var marker = this._getSelectionMarker();
        if (marker && marker.selectionLength) {
            var selection = this.props.text.substr(marker.selectionStart, marker.selectionLength);
            if (this.props.selectionHandler) {
                this.props.selectionHandler(selection, marker.selectionStart, marker.selectionLength);
            }
        }
    };
    InteractiveHighlighter.prototype.render = function () {
        this._segments = this._computeSegments();
        var textContent = this.props.text;
        var markedUp = [];
        for (var segmentNum = 0; segmentNum < this._segments.length; segmentNum++) {
            var spanStart = this._segments[segmentNum].start;
            var spanLen = this._segments[segmentNum].end - this._segments[segmentNum].start + 1;
            if (this._segments[segmentNum].highlights.length > 0) {
                // highlighted
                markedUp.push(react_1.default.createElement("span", { key: "d" + segmentNum, "data-segment": segmentNum, className: this._customClassFn(this._segments[segmentNum].highlights) }, textContent.substr(spanStart, spanLen)));
            }
            else {
                // non-highlighted
                markedUp.push(react_1.default.createElement("span", { key: "d" + segmentNum, "data-segment": segmentNum, className: this._customClassFn(this._segments[segmentNum].highlights) }, textContent.substr(spanStart, spanLen)));
            }
        }
        return (react_1.default.createElement("span", { onMouseUp: this.onMouseUpHandler }, markedUp));
    };
    return InteractiveHighlighter;
}(react_1.Component));
exports.InteractiveHighlighter = InteractiveHighlighter;
//# sourceMappingURL=InteractiveHighlighter.js.map